@@ -3,25 +3,49 @@
  Licensed under GPLv3
 """
 
-import subprocess
+import subprocess, os, shutil
+from urllib import request
+from shlex import quote
+
+
+__PACMAN_BIN = shutil.which("pacman")   # default to use the system's pacman binary
+
+
+def get_bin():
+    '''
+    Return the current pacman binary being used.
+    '''
+    return __PACMAN_BIN
+
+
+def set_bin(path):
+    '''
+    Set a custom pacman binary. 
+    If the pacman binary is set to an AUR helper, this module may also be used to interact with AUR.
+    '''
+    global __PACMAN_BIN
+    if isinstance(path, str) and (os.path.isfile(path) or os.path.isfile(shutil.which(path))):
+        __PACMAN_BIN = shutil.which(path)
+    else:
+        raise IOError("This executable does not exist.")
 
 
 def install(packages, needed=True):
-    # Install package(s)
+    '''Install package(s)'''
     s = pacman("-S", packages, ["--needed" if needed else None])
     if s["code"] != 0:
         raise Exception("Failed to install: {0}".format(s["stderr"]))
 
 
 def refresh():
-    # Refresh the local package information database
+    '''Refresh the local package information database'''
     s = pacman("-Sy")
     if s["code"] != 0:
         raise Exception("Failed to refresh database: {0}".format(s["stderr"]))
 
 
-def upgrade(packages=[]):
-    # Upgrade packages; if unspecified upgrade all packages
+def upgrade(packages=None):
+    '''Upgrade packages; if unspecified upgrade all packages'''
     if packages:
         install(packages)
     else:
@@ -31,14 +55,14 @@ def upgrade(packages=[]):
 
 
 def remove(packages, purge=False):
-    # Remove package(s), purge its files if requested
+    '''Remove package(s), purge its files if requested'''
     s = pacman("-Rc{0}".format("n" if purge else ""), packages)
     if s["code"] != 0:
         raise Exception("Failed to remove: {0}".format(s["stderr"]))
 
 
 def get_all():
-    # List all packages, installed and not installed
+    '''List all packages, installed and not installed'''
     interim, results = {}, []
     s = pacman("-Q")
     if s["code"] != 0:
@@ -54,7 +78,7 @@ def get_all():
             "installed": True
         }
     s = pacman("-Sl")
-    if s["code"] != 0 and s["stderr"]:
+    if s["code"] != 0:
         raise Exception(
             "Failed to get available list: {0}".format(s["stderr"])
         )
@@ -77,7 +101,7 @@ def get_all():
 
 
 def get_installed():
-    # List all installed packages
+    '''List all installed packages'''
     interim = {}
     s = pacman("-Q")
     if s["code"] != 0:
@@ -93,7 +117,7 @@ def get_installed():
             "installed": True
         }
     s = pacman("-Qu")
-    if s["code"] != 0:
+    if s["code"] != 0 and s["stderr"]:
         raise Exception(
             "Failed to get upgradable list: {0}".format(s["stderr"])
         )
@@ -113,7 +137,7 @@ def get_installed():
 
 
 def get_available():
-    # List all available packages
+    '''List all available packages'''
     results = []
     s = pacman("-Sl")
     if s["code"] != 0:
@@ -129,9 +153,9 @@ def get_available():
 
 
 def get_info(package):
-    # Get package information from database
+    '''Get package information from database'''
     interim = []
-    s = pacman("-Qi" if is_installed(package) else "-Si", package)
+    s = pacman("-Qi" if is_installed(package) else "-Si", package, pacman_bin=get_bin())
     if s["code"] != 0:
         raise Exception("Failed to get info: {0}".format(s["stderr"]))
     for x in s["stdout"].split('\n'):
@@ -171,15 +195,38 @@ def is_installed(package):
     return pacman("-Q", package)["code"] == 0
 
 
-def pacman(flags, pkgs=[], eflgs=[]):
-    # Subprocess wrapper, get all data
+def is_aur(package):
+    '''
+    Return True if the given package is an AUR package.
+    '''
+    try:
+        # search in official pacman repo
+        matched_packages = pacman('-Ssq', package, pacman_bin="pacman").get('stdout').split('\n')
+        for i in matched_packages:
+            if i == package:
+                # find a match in official repo. not aur.
+                return False
+
+        req = request.Request("https://aur.archlinux.org/packages/?O=0&SeB=N&K={}&outdated=&SB=n&SO=a&PP=50&do_Search=Go".format(package), data=b'')
+        res = request.urlopen(req)
+        data = res.read()
+        if b"No packages matched your search criteria." in data:
+            return False
+        return True
+
+    except Exception as e:
+        return False
+
+
+def pacman(flags, pkgs=[], eflgs=[], pacman_bin=__PACMAN_BIN):
+    '''Subprocess wrapper, get all data'''
     if not pkgs:
-        cmd = ["pacman", "--noconfirm", flags]
+        cmd = [pacman_bin, "--noconfirm", flags]
     elif type(pkgs) == list:
-        cmd = ["pacman", "--noconfirm", flags]
-        cmd += pkgs
+        cmd = [pacman_bin, "--noconfirm", flags]
+        cmd += [quote(s) for s in pkgs]
     else:
-        cmd = ["pacman", "--noconfirm", flags, pkgs]
+        cmd = [pacman_bin, "--noconfirm", flags, pkgs]
     if eflgs and any(eflgs):
         eflgs = [x for x in eflgs if x]
         cmd += eflgs
